/*
    dsconv.cc

    Utility task that converts a string column containing angles or timetags
    into a real valued column or vice versa.

    (c) 1999 European Space Agency - written by Marco Beijersbergen
*/

#include <param.h>		// SAS
#include <Dal.h>
#include <Xmm.h>
#include <CalOalUtility.h>
#include "Celestial.h"

#include <iostream>		// C++

#include "dsconv.h"

void dsconv()
{
  string colname = stringParameter("column");
  string typ = stringParameter("type");
  bool decode = (stringParameter("dir") == "decode");

  DataSet* set = dataSetServer->open(colname,Dal::Modify);
  Table* tab = set->table(colname);
  Column* oldcol = tab->column(colname);
  ColumnData *olddata = oldcol->data();

  string newcolname = stringParameter("newcolumn");
  Column* newcol;

  if (decode) {
    char* s = olddata->stringData();

    if (tab->hasColumn(newcolname))
      newcol = tab->column(newcolname);
    else
      newcol = tab->addColumn(newcolname,Dal::Real64);

    ColumnData *newdata = newcol->data();
    double* d = newdata->real64Data();
    for (unsigned i = 0; i < tab->rows(); ++i) {
      string datum(s+olddata->position(i),olddata->size()[0]);
      if (typ == "time")
        d[i] = Xmm::toTimeTag(datum);
      else if (typ == "ra")
	d[i] = RightAscension(datum).angle();
      else if (typ == "dec")
	d[i] = Declination(datum).angle();
      else
	errstr << "invalid decode type " << typ << Sas::error("invalidConversionType");
      errstr << "row " << i << ": " << datum << " -> " << d[i] << Sas::message(Msg::App,Msg::Noisy);
    }
    delete newdata;
  }
  else {
    double *d = olddata->real64Data();

    if (tab->hasColumn(newcolname))
      newcol = tab->column(newcolname);
    else
      newcol = tab->addColumn(newcolname,Dal::DString);

    ColumnData *newdata = newcol->data();
    char *s = newdata->stringData();
    for (unsigned i = 0; i < tab->rows(); ++i) {
      string datum;
      if (typ == "time")
        datum = STime(d[i]).toString();
      else if (typ == "ra")
        datum = RightAscension(d[i]).toString();
      else if (typ == "dec")
        datum = Declination(d[i]).toString();
      else
	errstr << "invalid encode type " << typ << Sas::error("invalidConversionType");

      strncpy(s+newdata->position(i),datum.c_str(),newdata->size()[0]);
    }
    delete newdata;
  }

  delete olddata;

  dataSetServer->close(set);
}
