/* written by John Doug Reynolds, September 2001 */

#include "regionmask.h"
#include <ShapeScanner.h>
#include <rgsPixelsLib.h>
#include <ASCregion.h>

using namespace rgsPixelsLib;

static GridSpan gridspan( int rpx, float rvl, float dlt, int len )
{
   return GridSpan( Grid(dlt,rvl-dlt*rpx), Span(rpx).length(len) );
}

static GridSpan gridspan( const string &column )
{
   struct DataSet *dataset = dataSetServer->open( column, Dal::Read );
   const Column *c = dataset->table(column)->column(column);

   float dlt = c->attribute("TCDLT")->asReal();
   float rvl = c->attribute("TCRVL")->asReal();
   int   rpx = c->attribute("TCRPX")->asInt();
   int   min = c->attribute(c->hasAttribute("TLMIN")?"TLMIN":"TDMIN")->asInt();
   int   max = c->attribute(c->hasAttribute("TLMAX")?"TLMAX":"TDMAX")->asInt();

   struct GridSpan gs = GridSpan( Grid(dlt,rvl-dlt*rpx), Span(min,max) );
   dataSetServer->close( dataset );
   return gs;
}

void regionmask()
{
   string regname = stringParameter("region");
   bool autosize = booleanParameter("autosize");

   string maskset;
   bool withmaskset = booleanParameter("withmaskset");
   if ( withmaskset )
      maskset = stringParameter("maskset");

   struct GridSpan xgrid, ygrid;
   string whichpixdef = stringParameter("whichpixdef");
   if ( whichpixdef == "image" ) {
      string pixdefset = stringParameter("pixdefset");
      struct DataSet *dataset = dataSetServer->open( pixdefset, Dal::Read );
      const Array *array = dataset->array("PRIMARY");
      xgrid = gridspan( array, 1 );
      ygrid = gridspan( array, 2 );
      dataSetServer->close( dataset );
   }
   else if ( whichpixdef == "columns" ) {
      xgrid = gridspan( stringParameter("xcolumn") );
      ygrid = gridspan( stringParameter("ycolumn") );
   }
   else if ( whichpixdef == "manual" ) {
      xgrid = gridspan( intParameter("xrpix"), realParameter("xrval"),
			realParameter("xdelt"), intParameter("xbins") );
      ygrid = gridspan( intParameter("yrpix"), realParameter("yrval"),
			realParameter("ydelt"), intParameter("ybins") );
   }
   else
      errstr << "unexpected parameter value --whichpixdef=" << whichpixdef
	     << Sas::error("InternalError");

   //

   struct GeoRegion region;
   InASCregionTable table( regname );
   table >> region;

   struct RegionScanner scanner( region, xgrid, ygrid );
   struct Bounds clip = autosize ? scanner.bounds() : Bounds(xgrid,ygrid);
   scanner.reset(&clip);

   int b;
   list<Span> spans;
   list<Span>::const_iterator itr;
   if ( withmaskset ) {
      struct DataSet *dataset = dataSetServer->open( maskset, Dal::Create );

      longVec size(2);
      size[0] = clip.row.length();
      size[1] = clip.col.length();

      struct Array *array = dataset->addArray( "PRIMARY", Dal::Int8, size );

#define SET(name,value,label) *array->addAttribute(name,label) = value

      SET( "CRPIX1", 1, "reference channel coordinate" );
      SET( "CRVAL1", xgrid[clip.col.min], "reference physical coordinate" );
      SET( "CDELT1", xgrid.stride, "uniform channel width" );

      SET( "CRPIX2", 1, "reference channel coordinate" );
      SET( "CRVAL2", ygrid[clip.row.min], "reference physical coordinate" );
      SET( "CDELT2", ygrid.stride, "uniform channel width" );

#undef SET

      struct ArrayData *arrayData = array->data();
      int8 *buffer = arrayData->int8Data();
      memset( buffer, 0, size[0] * size[1] * sizeof(int8) );

      struct PixelMap<int8> mask( clip, 0, buffer );
      while ( scanner.next(b,spans) )
	 for ( itr = spans.begin(); itr != spans.end(); ++itr )
	    mask.put( Bounds(Span(b),*itr), 1 );

      dataSetServer->close( dataset );
   }
   else
      while ( scanner.next(b,spans) )
	 for ( itr = spans.begin(); itr != spans.end(); ++itr )
	    cout << b << " " << itr->min << " " << itr->max << endl;
}
